      subroutine symexpand(nSymOp,SymMat,SymDis,nXatoms,nXmax,datX,
     +  zX,UniqueXAtom,natoms,maxatoms)
c
c  Routine to expand asymmetric unit to full unit cell for
c  program ZEOTS
c
c  by:
c  German Sastre     Instituto de Tecnologia Quimica.
c  Julian D. Gale    Imperial College.
c
c  Passed arguments :
c
c  integer nSymOp                    : number of symmetry operations
c  real*8  SymMat(3,3,maxsymop)      : symmetry rotation matrices
c  real*8  SymDis(3,maxsymop)        : symmetry displacement vector
c  integer nXatoms                   : total number of atoms of type X
c  integer nXmax                     : maximum number of atoms of type X
c  character*6 datX(maxatoms)        : labels for atoms of type X
c  real*8  zX(3,maxatoms)            : XYZ coordinates of atoms of type X
c  logical UniqueXAtom(maxatoms)     : logical - true => asymmetric unit X atom
c  integer natoms                    : total number of atoms (excluding H)
c  integer maxatoms                  : maximum number of atoms 
c
c  Local variables :
c
c  real*8  tol                       : tolerance for two atoms to be different
c
c ------------------------------------------------------------------------------

      implicit     none

c passed variables
      integer      natoms, maxatoms
      integer      nXatoms, nXmax
      integer      nSymOp

      real*8       zX(3,nXmax)
      real*8       SymMat(3,3,nSymOp),SymDis(3,nSymOp)

      logical      UniqueXAtom(nXmax)

      character*6  datX(nXmax)

c local variables
      integer      i, j, nop, initialnXatoms

      logical      found

      real*8       xyz(3), tol, dtot, dx, dy, dz

      data tol / 2.0d-4 /

      initialnXatoms = nXatoms

c loop over atoms in asymmetric unit
      do i = 1,initialnXatoms

c ensure that fractional coordinates of atoms lie within the central unit cell
        zX(1,i) = mod(zX(1,i) + 10.0d0, 1.0d0)
        zX(2,i) = mod(zX(2,i) + 10.0d0, 1.0d0)
        zX(3,i) = mod(zX(3,i) + 10.0d0, 1.0d0)

c loop over symmetry operators applying each one in turn
        do nop = 1,nSymOp

c generate equivalent position
          xyz(1) = zX(1,i)
          xyz(2) = zX(2,i)
          xyz(3) = zX(3,i)

c apply rotation operator
          xyz(1) = SymMat(1,1,nop) * zX(1,i) + 
     +             SymMat(2,1,nop) * zX(2,i) + 
     +             SymMat(3,1,nop) * zX(3,i)
          xyz(2) = SymMat(1,2,nop) * zX(1,i) + 
     +             SymMat(2,2,nop) * zX(2,i) + 
     +             SymMat(3,2,nop) * zX(3,i)
          xyz(3) = SymMat(1,3,nop) * zX(1,i) + 
     +             SymMat(2,3,nop) * zX(2,i) + 
     +             SymMat(3,3,nop) * zX(3,i)

c apply translation operator
          xyz(1) = xyz(1) + SymDis(1,nop)
          xyz(2) = xyz(2) + SymDis(2,nop)
          xyz(3) = xyz(3) + SymDis(3,nop)

c ensure that fractional coordinates lie within the central unit cell
          xyz(1) = mod(xyz(1) + 10.0d0, 1.0d0)
          xyz(2) = mod(xyz(2) + 10.0d0, 1.0d0)
          xyz(3) = mod(xyz(3) + 10.0d0, 1.0d0)

c check to see whether this atom is equivalent to one that already exists
          found = .false.
          j = 0
          do while (j .lt. nXatoms .and. .not.found)

            j = j + 1

            dx = abs(xyz(1) - zX(1,j))
            dy = abs(xyz(2) - zX(2,j))
            dz = abs(xyz(3) - zX(3,j))
            dx = min(dx, 1.0d0-dx)
            dy = min(dy, 1.0d0-dy)
            dz = min(dz, 1.0d0-dz)
            dtot = dx + dy + dz
 
            found = (dtot .lt. tol)

          enddo

c if atom hasn't been found, add to the list after checking array dimensions
          if (.not.found) then

            natoms = natoms + 1
            nXatoms = nXatoms + 1

            if (natoms .gt. maxatoms) then
              write(6,'(/,'' ERROR - Too many atoms for current '',
     +          ''program dimensions - increase maxatoms'',/)')
              stop
            endif
            if (nXatoms .gt. nXmax) then
              write(6,'(/,'' ERROR - Too many atoms for current '',
     +          ''program dimensions - increase maxatoms'',/)')
              stop
            endif

            datX(nXatoms) = datX(i)
            zX(1,nXatoms) = xyz(1)
            zX(2,nXatoms) = xyz(2)
            zX(3,nXatoms) = xyz(3)
            UniqueXatom(nXatoms) = .false.

          endif

        enddo

      enddo

      return

      end
