c ------------------------------------------------------------------------------
c                           zeoTsites
c                          version 1.2
c
c   calculates connectivity of all the T-sites of a .xtl file
c   vertex symbols, and performs various TOT geometry analysis
c   
c   input '.xtl'
c
c   by:
c   German Sastre     Instituto de Tecnologia Quimica.
c   Julian D. Gale    Imperial College.
c
c  Main variables used in program :
c
c  real*8  a                         : a cell parameter
c  real*8  b                         : b cell parameter
c  real*8  c                         : c cell parameter
c  real*8  alpha                     : alpha cell angle
c  real*8  beta                      : beta cell angle
c  real*8  gamma                     : gamma cell angle
c  real*8  adist                     : distance between two atoms squared
c  real*8  AngleTOT(maxnb,nTmax)     : T1-O-T2 angles
c  real*8  DistTO(maxnb,nTmax)       : T1-O distances
c  real*8  DistOT(maxnb,nTmax)       : O-T2 distances
c  real*8  DistTT(maxnb,nTmax)       : T1-T2 distances
c  real*8  carx                      : X Cartesian coordinate of an atom
c  real*8  cary                      : Y Cartesian coordinate of an atom
c  real*8  carz                      : Z Cartesian coordinate of an atom
c  real*8  cell(3,3)                 : unit cell vectors
c  character*6 dat(maxatoms)         : labels for all atoms
c  character*6 dato(nOmax)           : labels for oxygen atoms
c  integer imatrix(3,27)             : indicates the multiples of each
c                                    : lattice vector to be added to create
c                                    : each of the nearest 27 images of an
c                                    : atom due to periodicity
c  integer icon(maxnb,nshellmax,nTmax) : pointer to atom number of neighbour
c                                    : in a given coordination shell
c  integer iconox(maxnb,nTmax)       : pointer to oxygens bonded to T1
c  integer imatr(3,maxnb,nshellmax,nTmax) : cell index pointers for icon
c  integer ncon(nshellmax,nTmax)     : number of neighbours in a given shell
c  integer nconox(nTmax)             : number of oxygens bonded to T1
c  integer natoms                    : total number of atoms (excluding H)
c  integer nOatoms                   : total number of oxygen atoms
c  integer nTatoms                   : total number of tetrahedral atoms
c  integer nSymOp                    : number of symmetry operations
c  real*8  SymDis(3,maxsymop)        : symmetry displacement vector
c  real*8  SymMat(3,3,maxsymop)      : symmetry rotation matrices
c  logical UniqueAtom(maxatoms)      : logical - true => asymmetric unit atom
c  logical UniqueOAtom(nOmax)        : logical - true => asymmetric unit O atom
c  real*8  xicon(maxnb,nTmax)        : Cartesian x coordinate of T neighbour
c  real*8  yicon(maxnb,nTmax)        : Cartesian y coordinate of T neighbour
c  real*8  zicon(maxnb,nTmax)        : Cartesian z coordinate of T neighbour
c  real*8  xiconox(maxnb,nTmax)      : Cartesian x coordinate of O neighbour
c  real*8  yiconox(maxnb,nTmax)      : Cartesian y coordinate of O neighbour
c  real*8  ziconox(maxnb,nTmax)      : Cartesian z coordinate of O neighbour
c  real*8  z(3,maxatoms)             : XYZ coordinates of all atoms
c  real*8  zo(3,nOmax)               : XYZ coordinates of oxygen atoms
c  real*8  zzz(3,27)                 : holds the 27 images of the current atom
c ------------------------------------------------------------------------------

      implicit     none

      integer      maxatoms, nshellmax, maxsymop, maxMR, sqmaxnb
      integer      maxnb, maxnb2, maxword
      integer      nOmax, nTmax, nOTOmax, indO, indH
      integer      nlabT,nlabTO,nlabTT,nlabTOT,nlabOTO

c user adjustable parameters
c
c maxatoms  = maximum number of atoms in a unit cell
c nshellmax = maximum number of shells
c maxnb     = maximum number of immediate neighbours
c maxnb2    = maximum number of neighbours in a given shell
c maxsymop  = maximum number of symmetry operators
c
      parameter    ( maxatoms = 600 )
      parameter    ( nshellmax = 14 )
      parameter    ( maxMR = 14 )
      parameter    ( maxnb = 8 )
      parameter    ( maxnb2 = 1000 )
      parameter    ( maxsymop = 192 )
      parameter    ( maxword = 20 )

c dependent parameters
      parameter    ( nTmax = maxatoms/3 )
      parameter    ( nOmax = 2*nTmax )
      parameter    ( nOTOmax = maxnb*(maxnb-1)/2 )
      parameter    ( sqmaxnb = maxnb**2 )

      real*8       z(3,maxatoms),zzz(3,27),cell(3,3)
      real*8       zo(3,nOmax)
      real*8       t1oav(nTmax,nTmax),tav(nTmax)
      real*8       t2oav(nTmax,nTmax)
      real*8       t1ot2av(nTmax,nTmax),totav(nTmax)
      real*8       t1t2av(nTmax,nTmax),ttav(nTmax),otoav(nTmax)
      real*8       AngleTOT(maxnb,nTmax),DistTO(maxnb,nTmax)
      real*8       AngleOTO(nOTOmax,nTmax)
      real*8       DistOT(maxnb,nTmax),DistTT(maxnb,nTmax)
      real*8       xicon(maxnb,nTmax),xiconox(maxnb,nTmax)
      real*8       yicon(maxnb,nTmax),yiconox(maxnb,nTmax)
      real*8       zicon(maxnb,nTmax),ziconox(maxnb,nTmax)
      real*8       a,b,c,alpha,beta,gamma,pi
      real*8       adist,adist1,adist2,adist3,adist4
      real*8       adist12,adist22,adist32,adist42
      real*8       carx,cary,carz,cangle,tsite,dxyz
      real*8       xa,ya,za,xb,yb,zb,xc,yc,zc,xyzshift(3)
      real*8       xj(maxnb),yj(maxnb),zj(maxnb)
      real*8       xt1,yt1,zt1,xt2,yt2,zt2,xox,yox,zox
      real*8       floats(maxword),angoto(nTmax)
      real*8       DislabTO(maxatoms), DislabTT(maxatoms)
      real*8       AnglabTOT(maxatoms), AvelabT(maxatoms,4)
      real*8       AnglabOTO(maxatoms)
      real*8       SymMat(3,3,maxsymop),SymDis(3,maxsymop)
      real*8       vm2(sqmaxnb,nTmax)

      logical      UniqueAtom(maxatoms),UniqueOAtom(nOmax)

      integer      n12av(nTmax,nTmax),n1av(nTmax)
      integer      imatrix(3,27),nt(maxatoms),ntn(nTmax)
      integer      icom(maxnb2,nshellmax,nTmax)
      integer      iconox(maxnb,nTmax)
      integer      nats1(maxnb2),nats2(maxnb2)
      integer      ncon(nshellmax,nTmax), icon(maxnb2,nshellmax,nTmax)
      integer      imatr(3,maxnb2,nshellmax,nTmax)
      integer      atm1(maxnb2,nshellmax,5)
      integer      atm2(maxnb2,nshellmax,5)
      integer      nconox(nTmax),ioxy(maxnb,nTmax),ncon2(nTmax)
      integer      numlabT(maxatoms), numlabTO(maxatoms)
      integer      numlabTT(maxatoms), numlabTOT(maxatoms)
      integer      numOTO(nTmax), num2OTO(nTmax), num3OTO(nTmax)
      integer      numlabOTO(maxatoms)
      integer      vs(nOTOmax,nTmax),vm(nOTOmax,nTmax)
      integer      vs2(sqmaxnb,nTmax)
      integer      lpr1(nTmax,nTmax)
      integer      natoms,nTatoms,nOatoms
      integer      nTatomsin,nOatomsin,nSymOp
      integer      i,iii,j,k,l,m,mm,m1,m2,m3,n,nimage,nT1,nT2
      integer      iat,ico,in,jc,jc1,jc2,k1,k2,k2ox,k3,k4,k11
      integer      lcount,loop,nco,ij,ix,j1,j2
      integer      ix1,ix2,ix3,ix4,iy1,iy2,iy3,iy4
      integer      nflag,ind,ntt,l1,l2,nOTO,h,h1,h2,i1,i2
      integer      j1flag1,j1flag2,j2flag1,j2flag2,j3flag,j4flag,j5flag
      integer      nword,nfloat,nlen,min1,min2,min3
      integer      psoddring,nring,validring,shortcut,ringtype

      character*80 line,xtlfile
      character*32 doutfile
      character*30 words(maxword)
      character*20 dout, dtail
      character*6  dat(maxatoms), dato(nOmax), da
      character*6  dlabT(maxatoms), dT1, dT2
      character*12 dlabTO(maxatoms),dlabTT(maxatoms)
      character*12 dTO,dOT,dT1T2,dT2T1
      character*18 dlabTOT(maxatoms),dlabOTO(maxatoms)
      character*18 dT1OT2,dT2OT1,dO1TO2(nOTOmax),dO2TO1(nOTOmax)
      logical      eof,found

c these are fixed pieces of data
      data dtail   /'0.0000  1.0000  P  '/

c the following are pieces of data that the user may
c wish to change according to his/her own preferences
c
c minimum T-T distance
      data adist1  / 2.8d0 /
c maximum T-T distance
      data adist2  / 3.6d0 /
c minimum T-O distance
      data adist3  / 1.3d0 /
c maximum T-O distance
      data adist4  / 2.2d0 /

c initialise atom counters prior to input
      nTatoms = 0
      nOatoms = 0
      natoms  = 0

c initialise number of symmetry operators
      nSymOp  = 0

c initialise cell codes: e.g.100=cell at the right
      do nimage=1,27
        imatrix(1,nimage)=0
        imatrix(2,nimage)=0
        imatrix(3,nimage)=0
      enddo
      imatrix(1,2)=  1
      imatrix(1,3)= -1
      imatrix(2,4)=  1
      imatrix(2,5)= -1
      imatrix(3,6)=  1
      imatrix(3,7)= -1
      imatrix(1,8)=  1
      imatrix(2,8)=  1
      imatrix(1,9)=  1
      imatrix(2,9)= -1
      imatrix(1,10)=-1
      imatrix(2,10)= 1
      imatrix(1,11)=-1
      imatrix(2,11)=-1
      imatrix(1,12)= 1
      imatrix(3,12)= 1
      imatrix(1,13)= 1
      imatrix(3,13)=-1
      imatrix(1,14)=-1
      imatrix(3,14)= 1
      imatrix(1,15)=-1
      imatrix(3,15)=-1
      imatrix(2,16)= 1
      imatrix(3,16)= 1
      imatrix(2,17)= 1
      imatrix(3,17)=-1
      imatrix(2,18)=-1
      imatrix(3,18)= 1
      imatrix(2,19)=-1
      imatrix(3,19)=-1
      imatrix(1,20)=1
      imatrix(2,20)=1
      imatrix(3,20)=1
      imatrix(1,21)=1
      imatrix(2,21)=1
      imatrix(3,21)=-1
      imatrix(1,22)=1
      imatrix(2,22)=-1
      imatrix(3,22)=1
      imatrix(1,23)=1
      imatrix(2,23)=-1
      imatrix(3,23)=-1
      imatrix(1,24)=-1
      imatrix(2,24)=1
      imatrix(3,24)=1
      imatrix(1,25)=-1
      imatrix(2,25)=1
      imatrix(3,25)=-1
      imatrix(1,26)=-1
      imatrix(2,26)=-1
      imatrix(3,26)=1
      imatrix(1,27)=-1
      imatrix(2,27)=-1
      imatrix(3,27)=-1

c define local variables
      pi=4.0d0*atan(1.0d0)

c read zeotype label for labelling the output file names
      write(6,'(/)')
      write(6,'('' Welcome to zeoTsites : '')')
      write(6,'(/)')
      write(6,'('' What is the zeotype ? '',$)')
      read(5,'(a20)') dout

c read the name of the xtl file
      write (6,'(/)')
      write (6,'('' Input file ? '',$)')
      read  (5,'(a80)') xtlfile

c open the xtl file
      open (9,file=xtlfile,status='old',form='formatted',err=100)

c open the output files
      doutfile=' '
      doutfile="Connecty_of_"//dout
      open(10,file=doutfile)
      doutfile=' '
      doutfile="T1_sites_of_"//dout
      open(11,file=doutfile)
      doutfile=' '
      doutfile="T2_sites_of_"//dout
      open(12,file=doutfile)
      doutfile=' '
      nlen=index(dout,' ')-1
      doutfile="output_"//dout(1:nlen)//".xtl"
      open(13,file=doutfile)

c write headings to the output files
      write (10,'(''zeoTsites (version 1.2) generated the'',
     + '' output file'',/)')
      write (10,'(''Connectivity and geometry of '',a20,/)')dout
      write (11,'(''zeoTsites (version 1.2) generated the''
     + '' output file'')')
      write (11,'(''this file contains 1st neighbours of T sites'',
     +  '' in the following format:'')')
      write (11,'(''atom number:X1   atom name:X2   '',
     +  ''1st connectivity:X3  (1 line)'')')
      write (11,'(''atom number:X1j  atom name:X2j  '',
     +  ''cell code:X3j        (j=1,2,...X3 lines)''/)')
      write (12,'(''zeoTsites (version 1.2) generated the''
     + '' output file'')')
      write (12,'(''this file contains 2nd neighbours of T sites'',
     +  '' in the following format:'')')
      write (12,'(''atom number:X1   atom name:X2   '',
     +  ''2nd connectivity:X3  (1 line)'')')
      write (12,'(''atom number:X1j  atom name:X2j  '',
     +  ''cell code:X3j        (j=1,2,...X3 lines)''/)')

c read xtl file and parse relevant input data
      eof = .false.
      do while (.not.eof)

c read the next line
        read  (9,'(a80)',err=99,end=99) line

c read and write title + column headings
        if (index(line,'TITLE').eq.1) then
          write (10,'(a80,/)') line
          write (13,'(a80)') line
        endif

c read unit cell
        if (index(line,'CELL').eq.1) then
          write (13,'(a4)') 'CELL'
          read  (9,'(a80)',err=101) line
          call linepro(line,nword,words,nfloat,floats,maxword)
          a=floats(1)
          b=floats(2)
          c=floats(3)
          alpha=floats(4)
          beta=floats(5)
          gamma=floats(6)
          write(13,'(6f11.6)') a,b,c,alpha,beta,gamma
          write(10,'(a17)') 'Cell parameters: '
          write(10,'(6f11.6)') a,b,c,alpha,beta,gamma
        endif

c read atomic coordinates (only T and O atoms expected)
        if (index(line,'ATOM').eq.1) then
          read  (9,'(a80)',err=101) line
          write (13,'(/,a5)') 'ATOMS'
          write (13,'(a34,a38)') 
     +       'NAME       X          Y          Z'
     +      ,'    POT  CHARGE   TEMP    OCCUP  SCAT'
          do while (.not.eof)
            read  (9,'(a80)',end=99,err=101) line
            if (index(line,'EOF').ne.0) goto 99
            call linepro(line,nword,words,nfloat,floats,maxword)
            da=words(1)
            natoms = natoms + 1
c check number of atoms dimension
            if (natoms .gt. maxatoms) then
              write(6,'(/,'' ERROR - Too many atoms for current '',
     +          ''program dimensions - increase maxatoms'',/)')
              stop
            endif

c atoms other than oxygen (O) and hydrogen (H)
c are taken as T-atoms
            indO = index(da,'O')
            if (indO.eq.0) indO = index(da,'o')
            indH = index(da,'H')
            if (indH.eq.0) indH = index(da,'h')
            if (indH.gt.0.and.indH.lt.indO) indO = 0
            if (indO .gt. 0) then
              nOatoms = nOatoms + 1
              if (nOatoms .gt. nOmax) then
                write(6,'(/,'' ERROR - Too many oxygens for current '',
     +            ''program dimensions - increase maxatoms'',/)')
                stop
              endif
              dato(nOatoms)=da
              zo(1,nOatoms)=floats(1)
              zo(2,nOatoms)=floats(2)
              zo(3,nOatoms)=floats(3)
              UniqueOAtom(nOatoms) = .true.
            elseif (indH .gt. 0) then
              natoms = natoms - 1
            else
              nTatoms = nTatoms + 1
              if (nTatoms .gt. nTmax) then
                write(6,'(/,'' ERROR - Too many T-atoms for current '',
     +            ''program dimensions - increase maxatoms'',/)')
                stop
              endif
              dat(nTatoms)=da
              z(1,nTatoms)=floats(1)
              z(2,nTatoms)=floats(2)
              z(3,nTatoms)=floats(3)
              UniqueAtom(nTatoms) = .true.
            endif
          enddo
        endif

c read symmetry operator
        if (index(line,'SYM MAT').eq.1) then
          write (13,'(a80)') line
          call linepro(line,nword,words,nfloat,floats,maxword)
          if (nfloat.lt.12) then
            write(6,'(/,'' ERROR - Insufficient data for symmetry '',
     +        ''operator'',/)')
            stop
          endif
          nSymOp = nSymOp + 1
          if (nSymOp .gt. maxsymop) then
            write(6,'(/,'' ERROR - Too many symmetry operators for '',
     +        ''program dimensions - increase maxsymop'',/)')
            stop
          endif
          SymMat(1,1,nSymOp) = floats(1)
          SymMat(2,1,nSymOp) = floats(2)
          SymMat(3,1,nSymOp) = floats(3)
          SymMat(1,2,nSymOp) = floats(4)
          SymMat(2,2,nSymOp) = floats(5)
          SymMat(3,2,nSymOp) = floats(6)
          SymMat(1,3,nSymOp) = floats(7)
          SymMat(2,3,nSymOp) = floats(8)
          SymMat(3,3,nSymOp) = floats(9)
          SymDis(1,nSymOp)   = floats(10)
          SymDis(2,nSymOp)   = floats(11)
          SymDis(3,nSymOp)   = floats(12)
        endif

c read space group
        if (index(line,'SYMMETRY').eq.1) then
          write (13,'(a80)') line
        endif

      enddo
   99 continue

c save the input numbers of atoms
      nTatomsin = nTatoms
      nOatomsin = nOatoms

c apply symmetry operators to asymmetric unit to generate full cell
      call symexpand(nSymOp,SymMat,SymDis,nTatoms,nTmax,dat,
     +  z,UniqueAtom,natoms,maxatoms)
      call symexpand(nSymOp,SymMat,SymDis,nOatoms,nOmax,dato,
     +  zo,UniqueOAtom,natoms,maxatoms)

c output atom number information
      write(10,'(/,42x,''   T '',4x,''   O '')')
      write(10,'(''Number of atoms in the asymmetric unit = '',
     +  i5,4x,i5)') nTatomsin,nOatomsin
      write(10,'(''Number of atoms in the full unit cell  = '',
     +  i5,4x,i5,/)') nTatoms,nOatoms
      write (10,'(/,a4,t9,a4)') 'atom', 'atom'
      write (10,'(a6,t9,a5,t19,a21,t70,a6,/)') 'number'
     +  ,'label','coordination sequence','T-type'

c convert angles from degrees to radians
      alpha=alpha*pi/180.0d0
      beta=beta*pi/180.0d0
      gamma=gamma*pi/180.0d0

c transform:  a,b,c,alpha,beta,gamma into crystallographic matrix
      cell(1,1)= a
      cell(1,2)= 0.0d0
      cell(1,3)= 0.0d0
      cell(2,1)= b*cos(gamma)
      cell(2,2)= b*sin(gamma)
      cell(2,3)= 0.0d0
      cell(3,1)= c*cos(beta)
      cell(3,2)= c*(cos(alpha)-cos(gamma)*cos(beta))/sin(gamma)
      cell(3,3)= c*sqrt(1-(cos(beta))**2-((cos(alpha)-cos(gamma)
     .  *cos(beta))/sin(gamma))**2)

c add O coordinates in matrix z after T coordinates:
c T coordinates go from 1-->nTatoms; 
c O coordinates go from nTatoms+1-->nTatoms+nOatoms
      do i=1,nOatoms
        dat(nTatoms+i)=dato(i)
        z(1,nTatoms+i)=zo(1,i)
        z(2,nTatoms+i)=zo(2,i)
        z(3,nTatoms+i)=zo(3,i)
        UniqueAtom(nTatoms+i)=UniqueOAtom(i)
      enddo

c first connectivity: each atom with itself
      do in=1,nTatoms
        icon(1,1,in)=in
        imatr(1,1,1,in)=0
        imatr(2,1,1,in)=0
        imatr(3,1,1,in)=0
        ncon(1,in)=1
      enddo

c create squared distances to avoid square rooting all the time
      adist12 = adist1*adist1
      adist22 = adist2*adist2
      adist32 = adist3*adist3
      adist42 = adist4*adist4

c transform unit cell coordinates (only T atoms!) into cartesian system
      do k1=1,nTatoms
        carx= z(1,k1)*cell(1,1) + z(2,k1)*cell(2,1) + z(3,k1)*cell(3,1)
        cary= z(1,k1)*cell(1,2) + z(2,k1)*cell(2,2) + z(3,k1)*cell(3,2)
        carz= z(1,k1)*cell(1,3) + z(2,k1)*cell(2,3) + z(3,k1)*cell(3,3)
        k2=0
        k2ox=0
        do k3=1,natoms

c generate translational symmetry images of each atom in the border cells
c border cells= left,right,up,down,front,back,and diagonal cells (4* xy,xz,yz)
          do j=1,3
            zzz(j, 1)= z(1,k3)*cell(1,j) +
     +                 z(2,k3)*cell(2,j) +
     +                 z(3,k3)*cell(3,j)
          enddo
          do nimage = 2,27
            do j=1,3
              zzz(j,nimage)= zzz(j,1) + imatrix(1,nimage)*cell(1,j)
     +                                + imatrix(2,nimage)*cell(2,j)
     +                                + imatrix(3,nimage)*cell(3,j)
            enddo
          enddo

c distance: atom(k1)-[atom(k3) and its 26 symmetry translated images]
c repeat that process for each k1 (T) atom
c T neighbour if adist1 < dist < adist2 ...then calculate code number and cell
c O neighbour if adist3 < dist < adist4 ...then calculate code number and cell
          do nimage=1,27
            xt1 = carx - zzz(1,nimage)
            yt1 = cary - zzz(2,nimage)
            zt1 = carz - zzz(3,nimage)
            adist = (xt1*xt1+yt1*yt1+zt1*zt1)

c loop to test T-T connectivity...
c the code is careful: some T-O distances can be within the T-T
c range, thus check the second atom is not O: make sure is T-T
            if ((adist.gt.adist12).and.(adist.lt.adist22).
     +          and.(k3.le.nTatoms)) then
              k2=k2+1
              if (k2 .gt. maxnb) then
                write(6,'(/,'' ERROR - Too many neighbours for '',
     +            ''current dimensions - increase maxnb'',/)')
                stop
              endif
              icon(k2,2,k1)=k3
              imatr(1,k2,2,k1)=imatrix(1,nimage)
              imatr(2,k2,2,k1)=imatrix(2,nimage)
              imatr(3,k2,2,k1)=imatrix(3,nimage)
              ncon(2,k1)=k2
              xicon(k2,k1)=zzz(1,nimage)
              yicon(k2,k1)=zzz(2,nimage)
              zicon(k2,k1)=zzz(3,nimage)
            endif

c loop to test T-O connectivity
            if ((adist.gt.adist32).and.(adist.lt.adist42).
     +          and.(k3.gt.nTatoms)) then
              k2ox=k2ox+1
              if (k2ox .gt. maxnb) then
                write(6,'(/,'' ERROR - Too many neighbours for '',
     +            ''current dimensions - increase maxnb'',/)')
                stop
              endif
              iconox(k2ox,k1)=k3
              nconox(k1)=k2ox
              xiconox(k2ox,k1)=zzz(1,nimage)
              yiconox(k2ox,k1)=zzz(2,nimage)
              ziconox(k2ox,k1)=zzz(3,nimage)
            endif

          enddo
        enddo
      enddo

c calculate O connectivity: just find its two neighbours
c nT1 is the looping variable over the central T site (T1)
c nT2 is the looping variable over T sites adjacent to T1
      do nT1=1,nTatoms

c loop over T sites adjacent to T1
        do nT2=1,ncon(2,nT1)
          ico=icon(nT2,2,nT1)

c find unit cell shift of atom ico that is connected to T1
          do j=1,3
            xyzshift(j)= imatr(1,nT2,2,nT1)*cell(1,j)
     +                 + imatr(2,nT2,2,nT1)*cell(2,j)
     +                 + imatr(3,nT2,2,nT1)*cell(3,j)
          enddo

          k3=1
          found = .false.
          do while (k3.le.nconox(nT1).and..not.found)
            k4=1
            do while (k4.le.nconox(ico).and..not.found)
              if (iconox(k3,nT1).eq.iconox(k4,ico)) then

c additional check in case images are close
                dxyz = abs(xiconox(k3,nT1)-xiconox(k4,ico)-
     +                     xyzshift(1))+
     +                 abs(yiconox(k3,nT1)-yiconox(k4,ico)-
     +                     xyzshift(2))+
     +                 abs(ziconox(k3,nT1)-ziconox(k4,ico)-
     +                     xyzshift(3))
                if (dxyz.lt.1.0d-4) then
                  found = .true.
                  ioxy(nT2,nT1)=iconox(k3,nT1)
                  xox=xiconox(k3,nT1)
                  yox=yiconox(k3,nT1)
                  zox=ziconox(k3,nT1)
                endif
              endif
              k4=k4+1
            enddo
            k3=k3+1
          enddo

c now we have one TOT more. Let's calculate TOT angle...
c put coordinates of T1 O T2 in easy-to-handle variables
          xt1=z(1,nT1)*cell(1,1)+z(2,nT1)*cell(2,1)+z(3,nT1)*cell(3,1)
          yt1=z(1,nT1)*cell(1,2)+z(2,nT1)*cell(2,2)+z(3,nT1)*cell(3,2)
          zt1=z(1,nT1)*cell(1,3)+z(2,nT1)*cell(2,3)+z(3,nT1)*cell(3,3)
          xt2=xicon(nT2,nT1)
          yt2=yicon(nT2,nT1)
          zt2=zicon(nT2,nT1)
c our angle-vector scheme is: T1--O--T2
c vector T1-->O and its norm
          xa=xox-xt1
          ya=yox-yt1
          za=zox-zt1
          DistTO(nT2,nT1)=sqrt(xa*xa+ya*ya+za*za)
c vector O<--T2 and its norm
          xb=xox-xt2
          yb=yox-yt2
          zb=zox-zt2
          DistOT(nT2,nT1)=sqrt(xb*xb+yb*yb+zb*zb)
c T1-T2 distance
          xc=xt1-xt2
          yc=yt1-yt2
          zc=zt1-zt2
          DistTT(nT2,nT1)=sqrt(xc*xc+yc*yc+zc*zc)
c angle between two vectors
          cangle=(xa*xb+ya*yb+za*zb)/(DistTO(nT2,nT1)*DistOT(nT2,nT1))
          if (cangle.lt.-1) cangle=-1.0d0
          if (cangle.gt.1)  cangle=1.0d0
          AngleTOT(nT2,nT1)=acos(cangle)*180.0d0/pi
        enddo
c vectors O-->T
        do l=1,nconox(nT1)
          xj(l)=xt1-xiconox(l,nT1)
          yj(l)=yt1-yiconox(l,nT1)
          zj(l)=zt1-ziconox(l,nT1)
        enddo
c calculate Oi-T-Oj angles for each T-atom
        angoto(nT1)=0
        nOTO=0
        do l1=1,nconox(nT1)-1
          do l2=l1+1,nconox(nT1)
            cangle=xj(l1)*xj(l2)+yj(l1)*yj(l2)+zj(l1)*zj(l2)
            cangle=cangle/(DistTO(l1,nT1)*DistTO(l2,nT1))
            if (cangle.lt.-1) cangle=-1.0d0
            if (cangle.gt.1)  cangle=1.0d0
            nOTO=nOTO+1
            if (nOTO .gt. nOTOmax) then
              write(6,'(/,'' ERROR - Too many OTO angles for '',
     +        ''current dimensions. Maximum T-coordination has'',
     +        '' probably been exceeded - increase maxnb'',/)')
              stop
            endif
            AngleOTO(nOTO,nT1)=acos(cangle)*180.0d0/pi
            angoto(nT1)=angoto(nT1)+AngleOTO(nOTO,nT1)
          enddo
        enddo
        numOTO(nT1)=nOTO
        angoto(nT1)=angoto(nT1)/numOTO(nT1)
      enddo
c end of nT1-loop

c calculate all the other neighbours (jc=3, ... ,nshellmax)
      do jc=3,nshellmax
        do in=1,nTatoms
          lcount=0
          do l=1,ncon(jc-1,in)
            iii=icon(l,jc-1,in)
            do k=1,ncon(2,iii)
              lcount=lcount+1
              if (lcount .gt. maxnb2) then
                write(6,'(/,'' ERROR - Too many neighbours for '',
     +            ''current dimensions - increase maxnb2'',/)')
                stop
              endif
              iat=icon(k,2,icon(l,jc-1,in))
              icon(lcount,jc,in)=iat
              icom(lcount,jc,in)=l
              imatr(1,lcount,jc,in)=imatr(1,l,jc-1,in)+imatr(1,k,2,iii)
              imatr(2,lcount,jc,in)=imatr(2,l,jc-1,in)+imatr(2,k,2,iii)
              imatr(3,lcount,jc,in)=imatr(3,l,jc-1,in)+imatr(3,k,2,iii)

c is it repeated in shells 'jc-2, jc-1' ?. if so, discard it (nflag=1)
              nflag=0
              do n=jc-2,jc-1
                do m=1,ncon(n,in)
                  if ((icon(lcount,jc,in).eq.icon(m,n,in)).and.
     +                (imatr(1,lcount,jc,in).eq.imatr(1,m,n,in)).and.
     +                (imatr(2,lcount,jc,in).eq.imatr(2,m,n,in)).and.
     +                (imatr(3,lcount,jc,in).eq.imatr(3,m,n,in))) then
                    nflag=1
                    goto 33
                  endif
                enddo
              enddo

              if ( (lcount.gt.1).and.(nflag.eq.0) ) then
c is it repeated in shell 'jc' ?. if so, discard it (nflag=1)
                do m=1,lcount-1
                  if (  (icon(lcount,jc,in).eq.icon(m,jc,in)).and.
     +              (imatr(1,lcount,jc,in).eq.imatr(1,m,jc,in)).and.
     +              (imatr(2,lcount,jc,in).eq.imatr(2,m,jc,in)).and.
     +              (imatr(3,lcount,jc,in).eq.imatr(3,m,jc,in))) then
                    nflag=1
                  endif
                enddo
              endif

   33         if (nflag.eq.1) lcount=lcount-1

            enddo
          enddo

          ncon(jc,in)=lcount

        enddo
      enddo

c calculate number of topologically different T-types
      nt(1)=1
      ntt=1
      do in=2,nTatoms
        nflag=1
        loop=1
        do while ( (loop.lt.in).and.(nflag.ne.0) )
          nflag=0
          do nco=2,nshellmax
            if ( ncon(nco,in).ne.ncon(nco,loop) ) nflag=1
          enddo
          if (nflag.eq.0) nt(in)=nt(loop)
          loop=loop+1
        enddo
        if (nflag.ne.0) then
          ntt=ntt+1
          nt(in)=ntt
        endif
      enddo

c zero nt for oxygens
      do in=nTatoms+1,natoms
        nt(in) = 0.0d0
      enddo

c calculate multiplicity of each topologically different T-type
      do in=1,nTatoms
        ij=nt(in)
        ntn(ij)=ntn(ij)+1
      enddo

c calculate vertex symbols
      do in=1,nTatoms
        do k1=1,nconox(in)-1
        do k2=k1+1,nconox(in)
         j=k1*nconox(in)+k2

c now we are in jc=2
         jc=2
         atm1(1,jc,1)=icon(k1,2,in)
         atm1(1,jc,2)=imatr(1,k1,2,in)
         atm1(1,jc,3)=imatr(2,k1,2,in)
         atm1(1,jc,4)=imatr(3,k1,2,in)
         atm2(1,jc,1)=icon(k2,2,in)
         atm2(1,jc,2)=imatr(1,k2,2,in)
         atm2(1,jc,3)=imatr(2,k2,2,in)
         atm2(1,jc,4)=imatr(3,k2,2,in)
         nats1(jc)=1
         nats2(jc)=1
c next shell
         do while (jc.le.(maxMR/2))
          jc=jc+1

c find whether there is a common neighbour, k1(jc-1)-k2(jc-1)  (odd MR)
          nring=0
          do l1=1,nats1(jc-1)
          do l2=1,nats2(jc-1)
            psoddring=0
            validring=0
            do j1=1,ncon(2,atm1(l1,jc-1,1))
              ix1=icon(j1,2,atm1(l1,jc-1,1))
              ix2=imatr(1,j1,2,atm1(l1,jc-1,1))+atm1(l1,jc-1,2)
              ix3=imatr(2,j1,2,atm1(l1,jc-1,1))+atm1(l1,jc-1,3)
              ix4=imatr(3,j1,2,atm1(l1,jc-1,1))+atm1(l1,jc-1,4)
              if ( (ix1.eq.atm2(l2,jc-1,1)).and.
     +             (ix2.eq.atm2(l2,jc-1,2)).and.
     +             (ix3.eq.atm2(l2,jc-1,3)).and.
     +             (ix4.eq.atm2(l2,jc-1,4)) ) psoddring=1
            enddo
            if (psoddring.eq.1) then
              h2=l2
              do m2=jc-1,2,-1
                h1=l1
                do m1=jc-1,2,-1
                if ( (atm1(h1,m1,1).eq.atm2(h2,m2,1)).and.
     +               (atm1(h1,m1,2).eq.atm2(h2,m2,2)).and.
     +               (atm1(h1,m1,3).eq.atm2(h2,m2,3)).and.
     +               (atm1(h1,m1,4).eq.atm2(h2,m2,4)) ) validring=1
                h1=atm1(h1,m1,5)
                enddo
                h2=atm2(h2,m2,5)
              enddo
            endif
            if ((psoddring.eq.1).and.(validring.eq.0)) then
              shortcut=0
              h1=l1
              h2=l2
              ringtype=1
              call brevior(in,jc,j,l1,l2,ncon,icon,imatr,atm1,atm2,
     +                     shortcut,ringtype)
              if (shortcut.eq.1) goto 44
              call brevior(in,jc,j,l2,l1,ncon,icon,imatr,atm2,atm1,
     +                     shortcut,ringtype)
              if (shortcut.eq.1) goto 44
   44         if  (((vs2(j,in).ge.(2*jc-3)).or.(vs2(j,in).eq.0)).and.
     +            (shortcut.eq.0)) then
                vs2(j,in)=2*jc-3
                vm2(j,in)=vm2(j,in)+1
                nring=1
              endif
            endif
          enddo
          enddo
c if valid ring was found, not necessary to go to another 'jc'
          if (nring.eq.1) goto 55
c end of 'odd MR'

c generate shells 'jc' coming from k1, k2
          call shellgen(in,jc,icon,imatr,ncon,nats1,atm1)
          call shellgen(in,jc,icon,imatr,ncon,nats2,atm2)

c find whether there is a common link, k1(jc)-k2(jc)  (even MR)
          nring=0
          do l1=1,nats1(jc)
          do l2=1,nats2(jc)
            if ( (atm1(l1,jc,1).eq.atm2(l2,jc,1)).and.
     +           (atm1(l1,jc,2).eq.atm2(l2,jc,2)).and.
     +           (atm1(l1,jc,3).eq.atm2(l2,jc,3)).and.
     +           (atm1(l1,jc,4).eq.atm2(l2,jc,4)) ) then
              validring=0
              h2=atm2(l2,jc,5)
              do m2=jc-1,2,-1
                h1=atm1(l1,jc,5)
                do m1=jc-1,2,-1
                if ( (atm1(h1,m1,1).eq.atm2(h2,m2,1)).and.
     +               (atm1(h1,m1,2).eq.atm2(h2,m2,2)).and.
     +               (atm1(h1,m1,3).eq.atm2(h2,m2,3)).and.
     +               (atm1(h1,m1,4).eq.atm2(h2,m2,4)) ) validring=1
                h1=atm1(h1,m1,5)
                enddo
                h2=atm2(h2,m2,5)
              enddo
              if (validring.eq.0) then
                shortcut=0
                h1=atm1(l1,jc,5)
                h2=atm2(l2,jc,5)
                ringtype=0
                call brevior(in,jc,j,l1,l2,ncon,icon,imatr,atm1,atm2,
     +                       shortcut,ringtype)
                if (shortcut.eq.1) goto 48
                call brevior(in,jc,j,l2,l1,ncon,icon,imatr,atm2,atm1,
     +                       shortcut,ringtype)
                if (shortcut.eq.1) goto 48
   48           if  (((vs2(j,in).ge.(2*jc-2)).or.(vs2(j,in).eq.0)).and.
     +              (shortcut.eq.0)) then
                  vs2(j,in)=2*jc-2
                  vm2(j,in)=vm2(j,in)+1
                  nring=1
                endif
              endif
            endif
          enddo
          enddo
c if valid ring was found, not necessary to go to another 'jc'
          if (nring.eq.1) goto 55
c end of 'even MR'

         enddo
c end of 'do while'

   55    continue
        enddo
        enddo

c atom finished, now order vertex symbols
c first, vertex symbol ordering is by: ring size, ring multiplicity
c (no opposite angles are defined).
c this will hold for non 4-connected atoms
        if (nconox(in).ne.4) then
          do k=1,nconox(in)*(nconox(in)-1)/2
            min1=maxMR
            min2=maxMR
            do m=1,nconox(in)**2
              if ((vs2(m,in).lt.min1).and.(vs2(m,in).ne.0)) then
                min1=vs2(m,in)
                min2=vm2(m,in)
                min3=m
              endif
              if ((vs2(m,in).eq.min1).and
     +         .(vm2(m,in).lt.min2)) then
                min1=vs2(m,in)
                min2=vm2(m,in)
                min3=m
              endif
            enddo
            vs(k,in)=min1
            vm(k,in)=min2
            vs2(min3,in)=0
            vm2(min3,in)=0
          enddo
        else
c when atom is 4-connected the vertex symbol ordering criteria is by:
c ring size, opposite angles, ring multiplicity
          do k=1,5,2
            min1=maxMR
            min2=maxMR
            do m=1,nconox(in)**2
              if ((vs2(m,in).lt.min1).and.(vs2(m,in).ne.0)) then
                min1=vs2(m,in)
                min2=vm2(m,in)
                min3=m
              elseif ((vs2(m,in).eq.min1).and
     +         .(vm2(m,in).lt.min2)) then
                min1=vs2(m,in)
                min2=vm2(m,in)
                min3=m
              endif
            enddo
            vs(k,in)=min1
            vm(k,in)=min2
            vs2(min3,in)=0
            vm2(min3,in)=0
            if ((min3.eq.6).or.(min3.eq.16)) then
              vs(k+1,in)=vs2(22-min3,in)
              vm(k+1,in)=vm2(22-min3,in)
              vs2(22-min3,in)=0
              vm2(22-min3,in)=0
            else
              vs(k+1,in)=vs2(19-min3,in)
              vm(k+1,in)=vm2(19-min3,in)
              vs2(19-min3,in)=0
              vm2(19-min3,in)=0
            endif
          enddo
c test pair ordering
          do j=1,2
          do i=5,3,-2
            if (((vs(i,in).eq.vs(i-2,in)).and.(vm(i,in).eq.vm(i-2,in)))
     +           .and.(
     +       ((vs(i+1,in).eq.vs(i-1,in)).and.(vm(i+1,in).lt.vm(i-1,in)))
     +           .or.  (vs(i+1,in).lt.vs(i-1,in)) ) )  then
              j1=vs(i-2,in)
              j2=vm(i-2,in)
              vs(i-2,in)=vs(i,in)
              vm(i-2,in)=vm(i,in)
              vs(i,in)=j1
              vm(i,in)=j2
              j1=vs(i-1,in)
              j2=vm(i-1,in)
              vs(i-1,in)=vs(i+1,in)
              vm(i-1,in)=vm(i+1,in)
              vs(i+1,in)=j1
              vm(i+1,in)=j2
            endif
          enddo
          enddo
        endif
      enddo

c output
      do in=1,nTatoms
        if (UniqueAtom(in)) then
          write (10,'(t1,i3,t9,a6,t16,13i4,t72,i2)') in,dat(in),
     +      (ncon(j,in),j=2,nshellmax),nt(in)
        endif
        write (11,'(a12,i3,a12,a6,a20,i3)') 'atom number:', in
     +    ,'  atom name:',dat(in),'   1st connectivity:',ncon(2,in)
        write (12,'(a12,i3,a12,a6,a20,i3)') 'atom number:', in
     +    ,'  atom name:',dat(in),'   2nd connectivity:',ncon(3,in)
        do l=1,ncon(2,in)
          write (11,'(i5,a12,t20,3i3)') icon(l,2,in),
     +     dat(icon(l,2,in)),(imatr(j,l,2,in),j=1,3)
        enddo
        write (11,'(/)')
        do l=1,ncon(3,in)
          write (12,'(i5,a12,t20,3i3)') icon(l,3,in),
     +     dat(icon(l,3,in)),(imatr(j,l,3,in),j=1,3)
        enddo
        write (12,'(/)')
      enddo

      write (10,'(/,a42,i3,/)') 
     + 'Number of topologically different T-types:',ntt
      write (10,'(a28,12i3)') 'Multiplicity of each T-type:'
     + ,(ntn(j),j=1,ntt)

      write (10,'(/,a4,t9,a4)') 'atom', 'atom'
      write (10,'(a6,t9,a5,t19,a34)') 'number'
     +  ,'label','vertex symbol (multiplicity below)'
      do in=1,nTatoms
        l=nconox(in)*(nconox(in)-1)/2
        if (UniqueAtom(in)) then
          write (10,'(/,t1,i3,t9,a6,t16,15i4)')
     +      in,dat(in),(vs(j,in),j=1,l)
          write (10,'(t16,15i4)') (vm(j,in),j=1,l)
        endif
      enddo

c this is not necessary to print anymore: but keep it because is
c information useful at some point.
c       do in=1,nTatoms
c       write (10,'(a8,i3,a24,4i4,a21,i4)') 'T-atom: ',in
c    +  ,'  ...oxygens connected: '
c    +  ,iconox(1,in),iconox(2,in),iconox(3,in),iconox(4,in)
c    +  ,'  / Total connected: ',nconox(in)
c       enddo

      write (10,'(/,/,a26,/)') 'Geometry T1-O-T2 analysis:'
      write (10,'(a6,t9,a7,t18,a6,t27,a5,t36,a4,t44,a4,t51,a5
     +  ,t58,a7,t66,a6,t73,a7)')
     +  'T1type','T1-O-T2','T2type','T-O-T','T1-O','O-T2','T1-T2'
     +  ,'T1label','Olabel','T2label'
      write (10,'(/)')

      do k1=1,nTatoms
        n1av(k1) = 0
        otoav(k1) = 0.0d0
        tav(k1) = 0.0d0
        ttav(k1) = 0.0d0
        totav(k1) = 0.0d0
        do k2=1,nTatoms
          n12av(k2,k1) = 0
          lpr1(k2,k1) = 0
          t1oav(k2,k1) = 0.0d0
          t1t2av(k2,k1) = 0.0d0
          t1ot2av(k2,k1) = 0.0d0
          t2oav(k2,k1) = 0.0d0
        enddo
      enddo
      do k1=1,natoms
        AnglabOTO(k1) = 0.0d0
        AnglabTOT(k1) = 0.0d0
        AvelabT(k1,1) = 0.0d0
        AvelabT(k1,2) = 0.0d0
        AvelabT(k1,3) = 0.0d0
        AvelabT(k1,4) = 0.0d0
        DislabTO(k1) = 0.0d0
        DislabTT(k1) = 0.0d0
        numlabT(k1) = 0
        numlabTO(k1) = 0
        numlabTT(k1) = 0
        numlabTOT(k1) = 0
        numlabOTO(k1) = 0
      enddo

c T-O label pairs numbered by 'nlabTO',
c stored in 'dlabTO',
c T-O distance stored in 'DislabTO',
c initialise with first T-O.
c analogously: T, T-T, T-O-T
c
      nlabOTO=0
      nlabT=1
      nlabTO=1
      nlabTT=1
      nlabTOT=1
      dT1=dat(1)
      dTO=dat(1)//dat(ioxy(1,1))
      dT1T2=dat(1)//dat(icon(1,2,1))
      dT1OT2=dat(1)//dat(ioxy(1,1))//dat(icon(1,2,1))
      dlabT(nlabT)=dT1
      dlabTO(nlabTO)=dTO
      dlabTT(nlabTT)=dT1T2
      dlabTOT(nlabTOT)=dT1OT2

c start averages
c start k1-loop
      do k1=1,nTatoms
        dT1=dat(k1)
        ix1=nt(k1)
        do k11=1,numOTO(k1)
          otoav(ix1)=otoav(ix1)+AngleOTO(k11,k1)
          num2OTO(ix1)=numOTO(k1)
        enddo
c start k2-loop
        do k2=1,ncon(2,k1)
          lpr1(k1,icon(k2,2,k1))=1
          if ( lpr1(icon(k2,2,k1),k1).ne.1) then
            write (10,'(a1,i2,a1,3i4,a2,i2,a1,t25,f7.3,3f8.3
     +        ,t58,a6,t66,a6,t73,a6)')
     +        '(',nt(k1),')',k1,ioxy(k2,k1),icon(k2,2,k1),' ('
     +        ,nt(icon(k2,2,k1)),')',AngleTOT(k2,k1),DistTO(k2,k1)
     +        ,DistOT(k2,k1),DistTT(k2,k1)
     +        ,dat(k1),dat(ioxy(k2,k1)),dat(icon(k2,2,k1))
              ix2=nt(icon(k2,2,k1))
              t1oav(ix1,ix2)=t1oav(ix1,ix2)+DistTO(k2,k1)
              t2oav(ix1,ix2)=t2oav(ix1,ix2)+DistOT(k2,k1)
              t1ot2av(ix1,ix2)=t1ot2av(ix1,ix2)+AngleTOT(k2,k1)
              t1t2av(ix1,ix2)=t1t2av(ix1,ix2)+DistTT(k2,k1)
              tav(ix1)=tav(ix1)+DistTO(k2,k1)
              tav(ix2)=tav(ix2)+DistOT(k2,k1)
              totav(ix1)=totav(ix1)+AngleTOT(k2,k1)
              totav(ix2)=totav(ix2)+AngleTOT(k2,k1)
              ttav(ix1)=ttav(ix1)+DistTT(k2,k1)
              ttav(ix2)=ttav(ix2)+DistTT(k2,k1)
              n1av(ix1)=n1av(ix1)+1
              n1av(ix2)=n1av(ix2)+1
              n12av(ix1,ix2)=n12av(ix1,ix2)+1

c current labels are 'dTO', 'dOT': store them in 'dlabTO'
c but first find which matrix element of 'dlabTO' refers to them
c analogously: T, T-T, T-O-T
              dT2=dat(icon(k2,2,k1))
              dTO=dat(k1)//dat(ioxy(k2,k1))
              dOT=dat(icon(k2,2,k1))//dat(ioxy(k2,k1))
              dT1T2=dat(k1)//dat(icon(k2,2,k1))
              dT2T1=dat(icon(k2,2,k1))//dat(k1)
              dT1OT2=dat(k1)//dat(ioxy(k2,k1))//dat(icon(k2,2,k1))
              dT2OT1=dat(icon(k2,2,k1))//dat(ioxy(k2,k1))//dat(k1)
              j1flag1=0
              j1flag2=0
              j2flag1=0
              j2flag2=0
              j3flag=0
              j4flag=0
c T averages
c if no element of 'dlabT' contains 'dT1' or 'dT2': new matrix element (nlabT+1)
              do ind=1,nlabT
                if (dlabT(ind).eq.dT1) then
                  AvelabT(ind,1)=AvelabT(ind,1)+AngleTOT(k2,k1)
                  AvelabT(ind,2)=AvelabT(ind,2)+DistTO(k2,k1)
                  AvelabT(ind,3)=AvelabT(ind,3)+DistTT(k2,k1)
                  numlabT(ind)=numlabT(ind)+1
                  j1flag1=1
                endif
              enddo
              if (j1flag1.eq.0) then
                nlabT=nlabT+1
                dlabT(nlabT)=dT1
                AvelabT(nlabT,1)=AngleTOT(k2,k1)
                AvelabT(nlabT,2)=DistTO(k2,k1)
                AvelabT(nlabT,3)=DistTT(k2,k1)
                numlabT(nlabT)=numlabT(nlabT)+1
              endif
              do ind=1,nlabT
                if (dlabT(ind).eq.dT2) then
                  AvelabT(ind,1)=AvelabT(ind,1)+AngleTOT(k2,k1)
                  AvelabT(ind,2)=AvelabT(ind,2)+DistOT(k2,k1)
                  AvelabT(ind,3)=AvelabT(ind,3)+DistTT(k2,k1)
                  numlabT(ind)=numlabT(ind)+1
                  j1flag2=1
                endif
              enddo
              if (j1flag2.eq.0) then
                nlabT=nlabT+1
                dlabT(nlabT)=dT2
                AvelabT(nlabT,1)=AvelabT(nlabT,1)+AngleTOT(k2,k1)
                AvelabT(nlabT,2)=AvelabT(nlabT,2)+DistOT(k2,k1)
                AvelabT(nlabT,3)=AvelabT(nlabT,3)+DistTT(k2,k1)
                numlabT(nlabT)=numlabT(nlabT)+1
              endif
c T-O averages
c if no element of 'dlabTO' contains 'dTO' or 'dOT': 
c new matrix element (nlabTO+1)
              do ind=1,nlabTO
                if (dlabTO(ind).eq.dTO) then
                  DislabTO(ind)=DislabTO(ind)+DistTO(k2,k1)
                  numlabTO(ind)=numlabTO(ind)+1
                  j2flag1=1
                endif
              enddo
              if (j2flag1.eq.0) then
                nlabTO=nlabTO+1
                dlabTO(nlabTO)=dTO
                DislabTO(nlabTO)=DistTO(k2,k1)
                numlabTO(nlabTO)=numlabTO(nlabTO)+1
              endif
              do ind=1,nlabTO
                if (dlabTO(ind).eq.dOT) then
                  DislabTO(ind)=DislabTO(ind)+DistOT(k2,k1)
                  numlabTO(ind)=numlabTO(ind)+1
                  j2flag2=1
                endif
              enddo
              if (j2flag2.eq.0) then
                nlabTO=nlabTO+1
                dlabTO(nlabTO)=dOT
                DislabTO(nlabTO)=DistOT(k2,k1)
                numlabTO(nlabTO)=numlabTO(nlabTO)+1
              endif
c T-T averages
c if no element of 'dlabTT' contains 'dTT': new matrix element (nlabTT+1)
              do ind=1,nlabTT
                if ((dlabTT(ind).eq.dT1T2).or.
     +             (dlabTT(ind).eq.dT2T1)) then
                  DislabTT(ind)=DislabTT(ind)+DistTT(k2,k1)
                  numlabTT(ind)=numlabTT(ind)+1
                  j3flag=1
                endif
              enddo
              if (j3flag.eq.0) then
                nlabTT=nlabTT+1
                dlabTT(nlabTT)=dT1T2
                DislabTT(nlabTT)=DistTT(k2,k1)
                numlabTT(nlabTT)=numlabTT(nlabTT)+1
              endif
c T-O-T averages
c if no element of 'dlabTOT' contains 'dTOT': new matrix element (nlabTOT+1)
              do ind=1,nlabTOT
                if ((dlabTOT(ind).eq.dT1OT2).or.
     +             (dlabTOT(ind).eq.dT2OT1)) then
                  AnglabTOT(ind)=AnglabTOT(ind)+AngleTOT(k2,k1)
                  numlabTOT(ind)=numlabTOT(ind)+1
                  j4flag=1
                endif
              enddo
              if (j4flag.eq.0) then
                nlabTOT=nlabTOT+1
                dlabTOT(nlabTOT)=dT1OT2
                AnglabTOT(nlabTOT)=AngleTOT(k2,k1)
                numlabTOT(nlabTOT)=numlabTOT(nlabTOT)+1
              endif
          endif
        enddo
c close k2-loop
c OTO label averages need to pass only once every k1,
c that's why they are out of the k2-loop
        do ind=1,nlabT
          if (dlabT(ind).eq.dT1) then
            do k11=1,numOTO(k1)
             AvelabT(ind,4)=AvelabT(ind,4)+AngleOTO(k11,k1)
             num3OTO(ind)=numOTO(k1)
             ncon2(ind)=ncon(2,k1)
            enddo
          endif
        enddo
c O-T-O averages
        nOTO=0
        do l1=1,nconox(k1)-1
          do l2=l1+1,nconox(k1)
            nOTO=nOTO+1
            dO1TO2(nOTO)=dat(iconox(l1,k1))//
     +      dat(k1)//dat(iconox(l2,k1))
            dO2TO1(nOTO)=dat(iconox(l2,k1))//
     +      dat(k1)//dat(iconox(l1,k1))
          enddo
        enddo
        do k11=1,numOTO(k1)
          j5flag=0
          do ind=1,nlabOTO
            if ((dlabOTO(ind).eq.dO1TO2(k11)).or.
     +        (dlabOTO(ind).eq.dO2TO1(k11))) then
              AnglabOTO(ind)=AnglabOTO(ind)+AngleOTO(k11,k1)
              numlabOTO(ind)=numlabOTO(ind)+1
              j5flag=1
            endif
          enddo
c if no element of 'dlabOTO' contains 'dOTO': 
c new matrix element (nlabOTO+1)
          if (j5flag.eq.0) then
            nlabOTO=nlabOTO+1
            dlabOTO(nlabOTO)=dO1TO2(k11)
            AnglabOTO(nlabOTO)=AngleOTO(k11,k1)
            numlabOTO(nlabOTO)=numlabOTO(nlabOTO)+1
          endif
        enddo
c end of OTO label average, 
      enddo
c close k1-loop

c print O-T-O geometry analysis
      write (10,'(/,/,a26,/)') 'Geometry O1-T-O2 analysis:'
      write (10,'(t4,a2,t10,a1,t12,a6,t20,a2,t27,a5
     +  ,t36,a18,t58,a7,t66,a6,t73,a7)') 'O1','T','T-type'
     +  ,'O2','O-T-O','Average  (Oi-T-Oj)','O1label'
     +  ,'Tlabel','O2label'
      write (10,'(/)')

c print O-T-O angles of each k1-atom
      do k1=1,nTatoms
        nOTO=0
        do l1=1,nconox(k1)-2
          do l2=l1+1,nconox(k1)
            nOTO=nOTO+1
            write (10,'(t3,i3,i5,t13,a1,i2,a1,t19,i3,t25,f7.3
     +      ,t58,a6,t66,a6,t73,a6)')  iconox(l1,k1),k1
     +      ,'(',nt(k1),')',iconox(l2,k1),AngleOTO(nOTO,k1)
     +      ,dat(iconox(l1,k1)),dat(k1),dat(iconox(l2,k1))
          enddo
        enddo
        l1=nconox(k1)-1
        l2=nconox(k1)
        write (10,'(t3,i3,i5,t13,a1,i2,a1,t19,i3,t25,f7.3
     +  ,t47,f7.3,t58,a6,t66,a6,t73,a6)') iconox(l1,k1),k1
     +  ,'(',nt(k1),')',iconox(l2,k1),AngleOTO(nOTO+1,k1)
     +  ,angoto(k1),dat(iconox(l1,k1)),dat(k1),dat(iconox(l2,k1))
      enddo

c print averages
      write (10,'(/,/,a38,/)') 
     + 'Average topological geometry analysis:'
      write (10,
     +  '(a6,t18,a6,t27,a5,t36,a4,t44,a4,t51,a5,t60,a6,/)')
     +  'T1type','T2type','T-O-T','T1-O','O-T2','T1-T2','Number'
      do k3=1,nTmax
        do k4=1,nTmax
          if ( n12av(k3,k4).gt.0 ) then
            write (10,
     +        '(a1,i2,a1,t18,a1,i2,a1,t25,f7.3,3f8.3,t60,i6)')
     +        '(',k3,')','(',k4,')',t1ot2av(k3,k4)/n12av(k3,k4)
     +        ,t1oav(k3,k4)/n12av(k3,k4),t2oav(k3,k4)/n12av(k3,k4)
     +        ,t1t2av(k3,k4)/n12av(k3,k4),n12av(k3,k4)
          endif
        enddo
      enddo
      write (10,'(/,/,a6,t27,a5,t36,a4,t51,a5,t60,a6,/)')
     +  'T1type','T-O-T','T1-O','T1-T2','Number'
      do k3=1,nTmax
        if ( n1av(k3).gt.0 ) then
          write (10,
     +      '(a1,i2,a1,t25,f7.3,f8.3,t48,f8.3,t60,i6)')
     +      '(',k3,')',totav(k3)/n1av(k3),tav(k3)/n1av(k3)
     +      ,ttav(k3)/n1av(k3),n1av(k3)
        endif
      enddo
      write (10,'(/,/,a6,t27,a5,t60,a6,/)') 
     +  'T1type','O-T-O','Number'
      do ix1=1,ntt
      write (10,'(a1,i2,a1,t25,f7.3,t60,i6)') '(',ix1,')',
     +  otoav(ix1)/(ntn(ix1)*num2OTO(ix1)),ntn(ix1)*num2OTO(ix1)
      enddo

      write (10,'(/,/,a32,/)') 'Average label geometry analysis:'
      write (10,'(t2,a10,t36,a3,t60,a6,/)')
     +  'T-O-labels','T-O','Number'
      do k3=1,nlabTO
      write (10,'(t3,a12,t35,f5.3,t60,i6)')
     +  dlabTO(k3),DislabTO(k3)/numlabTO(k3),numlabTO(k3)
      enddo
      write (10,'(/,/,t2,a10,t51,a3,t60,a6,/)')
     +  'T-T-labels','T-T','Number'
      do k3=1,nlabTT
      write (10,'(t3,a12,t51,f5.3,t60,i6)')
     +  dlabTT(k3),DislabTT(k3)/numlabTT(k3),numlabTT(k3)
      enddo
      write (10,'(/,/,t2,a12,t27,a5,t60,a6,/)')
     +  'T-O-T-labels','T-O-T','Number'
      do k3=1,nlabTOT
      write (10,'(t3,a18,t25,f7.3,t60,i6)')
     +  dlabTOT(k3),AnglabTOT(k3)/numlabTOT(k3),numlabTOT(k3)
      enddo
      write (10,'(/,/,t2,a12,t27,a5,t60,a6,/)')
     +  'O-T-O-labels','O-T-O','Number'
      do k3=1,nlabOTO
      write (10,'(t3,a18,t25,f7.3,t60,i6)')
     +  dlabOTO(k3),AnglabOTO(k3)/numlabOTO(k3),numlabOTO(k3)
      enddo
      write (10,'(/,/,t2,a7,t27,a5,t36,a3,t51,a3,t60,a6,/)')
     +  'T-label','T-O-T','T-O','T-T','Number'
      do k3=1,nlabT
      write (10,'(t3,a6,t25,f7.3,f8.3,t48,f8.3,t60,i6)')
     +  dlabT(k3), AvelabT(k3,1)/numlabT(k3),
     +  AvelabT(k3,2)/numlabT(k3), AvelabT(k3,3)/numlabT(k3),
     +  numlabT(k3)
      enddo
      write (10,'(/,/,t2,a7,t27,a5,t60,a6,/)')
     +  'T-label','O-T-O','Number'
      do k3=1,nlabT
      write (10,'(t3,a6,t25,f7.3,t60,i6)') dlabT(k3), 
     +  AvelabT(k3,4)/(numlabT(k3)*num3OTO(k3)/ncon2(k3))
     +  ,numlabT(k3)*num3OTO(k3)/ncon2(k3)
      enddo

c write .xtl file containing same coords than input, but T topological 
c site (divided by 1000) in the charges column. 
c forcefield column and others are useless here.
c Only write output asymmetric unit atoms.
      do i=1,natoms
        if (UniqueAtom(i)) then
          tsite= real(nt(i))/1000.0d0
          write (13,9) dat(i),(z(j,i),j=1,3),"gs",tsite,dtail
        endif
      enddo

c write end of file for new .xtl file
      write (13,'(a3)') 'EOF'

      write(6,'(/,'' zeoTsites completed successfully '',/)')
      stop

    9 format (a6,t8,f8.5,t19,f8.5,t30,f8.5,t40,a2,t45,f6.4,t53,a20)

  100 write(6,'(/,'' ERROR - cannot open the XTL file '',a80,/)')xtlfile
      stop

  101 write(6,'(/,'' ERROR - premature end of XTL file'',/)')
      stop

      end
